import json
import logging
import uuid
from datetime import datetime

import requests

from carom.config import config

logger = logging.getLogger(__name__)


def singleton(the_class):
    """ decorator for a class to make a singleton out of it """
    class_instances = {}

    def get_instance(*args, **kwargs):
        """ creating or just return the one and only class instance.
            The singleton depends on the parameters used in __init__ """
        key = (the_class, args, str(kwargs))
        if key not in class_instances:
            class_instances[key] = the_class(*args, **kwargs)
        return class_instances[key]

    return get_instance


def get_uuid():
    return uuid.uuid5(uuid.NAMESPACE_DNS, str(uuid.getnode()))


def send_to_server(state, table):
    d = datetime.now()
    url = config['connection']['endpoint'].format('billard/api/v1/locationdata/')
    headers = {'Authorization': 'Token {}'.format(config['connection']['token']),
               'Content-Type': 'application/json'}
    payload = {'client_id': str(get_uuid()),
               'desk_no': str(table),
               'tst': d.strftime('%Y-%m-%dT%H:%M:%S%Z'),
               'on_off': state}
    logger.info("Send Status: {}, {}, {}".format(str(url), str(headers), str(payload)))
    requests.post(
        url,
        data=json.dumps(payload),
        headers=headers,
        verify=False,
    )
